@@ -15,13 +15,13 @@ use nextest_metadata::{BinaryListSummary, BuildPlatform};
 use nextest_runner::{
     config::{NextestConfig, NextestProfile},
     errors::{TargetRunnerError, WriteEventError},
+    list::{BinaryList, OutputFormat, RustTestArtifact, SerializableFormat, TestList},
     partition::PartitionerBuilder,
     reporter::{StatusLevel, TestOutputDisplay, TestReporterBuilder},
     runner::TestRunnerBuilder,
     signal::SignalHandler,
     target_runner::TargetRunner,
     test_filter::{RunIgnored, TestFilterBuilder},
-    test_list::{BinaryList, OutputFormat, RustTestArtifact, SerializableFormat, TestList},
 };
 use owo_colors::{OwoColorize, Style};
 use std::{
@@ -178,7 +178,7 @@ enum Command {
     /// For more information, see <https://nexte.st/book/running>.
     Run {
         /// Nextest profile to use
-        #[clap(long, short = 'P')]
+        #[clap(long, short = 'P', env = "NEXTEST_PROFILE")]
         profile: Option<String>,
 
         /// Run tests serially and do not capture output
@@ -269,7 +269,12 @@ struct TestBuildFilter {
     cargo_options: CargoOptions,
 
     /// Run ignored tests
-    #[clap(long, possible_values = RunIgnored::variants(), default_value_t, value_name = "WHICH")]
+    #[clap(
+        long,
+        possible_values = RunIgnored::variants(),
+        default_value_t,
+        value_name = "WHICH",
+    )]
     run_ignored: RunIgnored,
 
     /// Test partition, e.g. hash:1/2 or count:2/3
@@ -294,16 +299,25 @@ impl TestBuildFilter {
         runner: &TargetRunner,
         reuse_build: &ReuseBuildOpts,
     ) -> Result<TestList<'g>> {
-        let path_mapper = reuse_build.make_path_mapper(graph);
+        let path_mapper =
+            reuse_build.make_path_mapper(graph, &binary_list.rust_build_meta.target_directory);
+        let rust_build_meta = binary_list.rust_build_meta.clone();
         let test_artifacts = RustTestArtifact::from_binary_list(
             graph,
             binary_list,
-            path_mapper.as_ref(),
+            &path_mapper,
             self.platform_filter.into(),
         )?;
         let test_filter =
             TestFilterBuilder::new(self.run_ignored, self.partition.clone(), &self.filter);
-        TestList::new(test_artifacts, &test_filter, runner).wrap_err("error building test list")
+        TestList::new(
+            test_artifacts,
+            &rust_build_meta,
+            &path_mapper,
+            &test_filter,
+            runner,
+        )
+        .wrap_err("error building test list")
     }
 
     fn compute_binary_list(
@@ -348,7 +362,8 @@ pub struct TestRunnerOpts {
         short = 'j',
         visible_alias = "jobs",
         value_name = "THREADS",
-        conflicts_with = "no-capture"
+        conflicts_with = "no-capture",
+        env = "NEXTEST_TEST_THREADS"
     )]
     test_threads: Option<usize>,
 
@@ -393,7 +408,8 @@ struct TestReporterOpts {
         long,
         possible_values = TestOutputDisplay::variants(),
         conflicts_with = "no-capture",
-        value_name = "WHEN"
+        value_name = "WHEN",
+        env = "NEXTEST_FAILURE_OUTPUT",
     )]
     failure_output: Option<TestOutputDisplay>,
     /// Output stdout and stderr on success
@@ -402,13 +418,19 @@ struct TestReporterOpts {
         long,
         possible_values = TestOutputDisplay::variants(),
         conflicts_with = "no-capture",
-        value_name = "WHEN"
+        value_name = "WHEN",
+        env = "NEXTEST_SUCCESS_OUTPUT",
     )]
     success_output: Option<TestOutputDisplay>,
 
     // status_level does not conflict with --no-capture because pass vs skip still makes sense.
     /// Test statuses to output
-    #[clap(long, possible_values = StatusLevel::variants(), value_name = "LEVEL")]
+    #[clap(
+        long,
+        possible_values = StatusLevel::variants(),
+        value_name = "LEVEL",
+        env = "NEXTEST_STATUS_LEVEL",
+    )]
     status_level: Option<StatusLevel>,
 }
 
@@ -452,7 +474,11 @@ impl App {
 
         let graph_data = match reuse_build.cargo_metadata.as_deref() {
             Some(path) => std::fs::read_to_string(path)?,
-            None => acquire_graph_data(manifest_path.as_deref(), output)?,
+            None => acquire_graph_data(
+                manifest_path.as_deref(),
+                build_filter.cargo_options.target_dir.as_deref(),
+                output,
+            )?,
         };
         let graph = guppy::CargoMetadata::parse_json(&graph_data)?.build_graph()?;
 
@@ -604,17 +630,24 @@ impl App {
     }
 }
 
-fn acquire_graph_data(manifest_path: Option<&Utf8Path>, output: OutputContext) -> Result<String> {
+fn acquire_graph_data(
+    manifest_path: Option<&Utf8Path>,
+    target_dir: Option<&Utf8Path>,
+    output: OutputContext,
+) -> Result<String> {
     let mut cargo_cli = CargoCli::new("metadata", manifest_path, output);
     // Construct a package graph with --no-deps since we don't need full dependency
     // information.
     cargo_cli.add_args(["--format-version=1", "--all-features", "--no-deps"]);
 
+    let mut expression = cargo_cli.to_expression().stdout_capture().unchecked();
+    // cargo metadata doesn't support "--target-dir" but setting the environment
+    // variable works.
+    if let Some(target_dir) = target_dir {
+        expression = expression.env("CARGO_TARGET_DIR", target_dir);
+    }
     // Capture stdout but not stderr.
-    let output = cargo_cli
-        .to_expression()
-        .stdout_capture()
-        .unchecked()
+    let output = expression
         .run()
         .wrap_err("cargo metadata execution failed")?;
     if !output.status.success() {
